package de.hdm_stuttgart.mi.sd1.statistics;

import java.util.Arrays;

/**
 * A container holding a fixed
 *  number of integer values.
 *
 */
public class IntegerStore {
  
  final static int defaultCapacity = 4;
  private int[] values ;         // Array containing our values
  private int numValues = 0;     // Number of values present in the container so far.
  
  /**
   * Create a new store being able to
   * hold integer values.
   * 
   */
  public IntegerStore() {
    values = new int[defaultCapacity];
  }
  /**
   * Create a new store being able to
   * hold integer values.
   * 
   * @param capacity The store's initial capacity. If more values
   * are being added via {@link #addValue(int)} the capacity
   * will be increased dynamically.
   * 
   */
  public IntegerStore(int capacity) {
    values = new int[capacity];
  }

  /**
   * Initializing from a given array of integers.
   * 
   * @param values These presumably  unsorted values will be sorted
   *     when being added to this store.
   */
  public IntegerStore(final int[] values) {

    this.values = new int[values.length];
    for (final int value: values) {
      addValue(value);
    }   
  }

  
  /**
   * @return The number of elements the store may
   * hold, see {@link #IntegerStore(int)}.
   */
  public int getCapacity() {
    return values.length;
  }
  
  /**
   * @return The number of values being contained.
   */
  public int getNumValues() { 
    return numValues;
  }
  
  /**
   * Insert a new value into our container
   * 
   * @param value The value to be inserted. This will increment
   * {@link #getNumValues()} by one.
   * 
   */
  public void addValue(int value) {
    if (values.length <= numValues) { // Insufficient capacity available to add a new value?
      final int[] currentArray = values; // Save the current array variable
      values = new int[2 * currentArray.length]; // Double the current array's size.
      for (int i = 0; i < currentArray.length; i++) { // Copy old values to new array
        values[i] = currentArray[i];
      }
    } 
    int insertIndex;
    // Find the index of the first existing element
    // being larger or equal to the new element to be inserted
    for (insertIndex= 0; insertIndex < numValues && values[insertIndex] < value; insertIndex++);
    
    /*
     * Example insert value == 3; Finding the right position
     * and shift some values accordingly to create a "free"
     * position
     *  Index values     | 0| 1| 2| 3| 4| 5| ...
     *  -----------------+--+--+--+--+-----+ ...
     *  values oldArray  | 1| 2| 7| 9|  |  |  
     *  -----------------+--+--+--+--+-----+ ...
     *  shift right one index starting from value "7" and fill in value 3
     *  -----------------+--+--+--+--+-----+ ...
     *  values newArray  | 1| 2| 3| 7| 9|  | ...
     */
    
    for (int i = numValues; insertIndex <  i; i--) { // Shift values if necessary
      values[i] = values[i - 1];
    }
    values[insertIndex] = value;
    numValues++;
  }

  /**
   * Access the value at a given index
   * 
   * @param index The desired value's index
   * @return The desired value.
   * <dl>
      <dt><b>Precondition:</b></dt>
       <dd>index &lt; {@link #getNumValues()}</dd>
     </dl>
   * 
   */
  public int getValue(int index) {
    return values[index];
  }
  
  /**
   * @return The array of values entered so far
   */
  public int[] getValues() {
    return Arrays.copyOfRange(values, 0, numValues);
  }
  
  /**
   * Empty the current set of values and set the store
   * to its initial state.
   */
  public void clear() {
    numValues = 0;
  }
  
  /**
   * @return The sample's average value.
   */
  public double getAverage() {
    double sum = 0;
    for (int i = 0; i < numValues; i++) {
      sum += values[i];
    }
    return sum / numValues;
  }
  /**
   *<dl>
      <dt><b>Precondition:</b></dt>
       <dd>There must be at least one element.</dd>
     </dl>
   * 
   * @return The sample's median.
   */
  public double getMedian() {
    final int upperMiddleIndex = numValues / 2;
    if (0 == numValues % 2) { // Even number of values
      return (values[upperMiddleIndex] +    // Choose mean of both
          values[upperMiddleIndex -1]) / 2.;// center values
    } else {  // Odd number of values
      return values[upperMiddleIndex]; 
    }
  } 
}