package de.hdm_stuttgart.mi.sd1.textstat;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Count words and their respective frequencies of appearance.
 *
 */
public class TextFileHandler {

  /**
   * The input file name to read from.
   */
  public final String filename;
  
  /**
   * The list of words found so far among with their frequencies.
   */
  final List<WordFrequency> wordFrequencies = new ArrayList<WordFrequency>();
  
  /**
   * @param filename The filename input to read input, see {@link #filename}
   * 
   * @throws IOException Input file cannot be read
   * @throws FileNotFoundException File does not exist
   */
  public TextFileHandler(final String filename)
      throws FileNotFoundException, IOException {
    
    this.filename = filename;
    final BufferedReader reader = new BufferedReader(new FileReader(filename));
    for (String line = reader.readLine(); line != null; line = reader.readLine()) {
      // Ignore white space and some common text markers.
      final String[] words = line.split("[ \t\"!?.,'´`]+");
      for (final String word: words) {
        
        // Word might be new
        final WordFrequency newWordCandidate = new WordFrequency(word);
        final int existingIndex = wordFrequencies.indexOf(newWordCandidate);
        if (existingIndex < 0) { // Word is not yet in list.
          wordFrequencies.add(newWordCandidate);
        } else { // Word is already in our list. Just increment its frequency value.
          wordFrequencies.get(existingIndex).incrementFrequency();
        }
      }
    }
    reader.close();
    Collections.sort(wordFrequencies, new WordFrequency.CompareByFrequency());
  }
  
  /**
   * Print a list of all words sorted by their respective
   * frequencies.
   */
  public void printFrequencies() {
    for (final WordFrequency wordFrequency : wordFrequencies) {
      System.out.format("%3d: %s\n",
                         wordFrequency.getFrequency(),
                              wordFrequency.word );
    }
  }
}
