package sda.jdbc.intro.auth;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.SQLException;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

/**
 * Collecting user input to enter Person instances into
 * a database. Features:
 * <ul>
 *   <li>User authentication</li>
 *   <li>hashed passwords</li>
 * </ul>
 * @author goik
 *
 */
public class InsertPerson extends JFrame {
   
   private static final long serialVersionUID = 6815975741605247675L;
   
   final PersistenceHandler persistenceHandler = new PersistenceHandler();
   
   // GUI attributes for user input
   final UserInputUnit name =
         new UserInputUnit(
               "Name",
               new RegexpVerifier(new JTextField(15), "^[^;'\"]+$", "No special characters allowed"));

   final UserInputUnit email =
         new UserInputUnit(
               "Email",
               new RegexpVerifier(new JTextField(20), "^[\\w\\-\\.\\_]+@[\\w\\-\\.]*[a-zA-Z]{2,4}$", "email not valid"));

   final UserInputUnit login =
         new UserInputUnit(
               "Login",
               new RegexpVerifier(new JTextField(10), "^[a-z][a-z0-9]*$", "login name not valid"));

   // We need a reference to the password field to avoid
   // casting from JTextField later.
   private final JPasswordField passwordField = new JPasswordField(10);
   private final UserInputUnit password =
         new UserInputUnit(
               "Password",
               new RegexpVerifier(passwordField, "^.{6,20}$", "length from 6 to 20 characters"));

   private final UserInputUnit passwordRepeat =
         new UserInputUnit(
               "repeat pass.",
               new EqualValueVerifier(new JPasswordField(10), passwordField, "Passwords do not match"));
   
   private final UserInputUnit [] userInputUnits =
      {name, email, login, password, passwordRepeat};


   private final JButton toggleConnectButton = new JButton(),
                        insertButton = new JButton("Insert");

   private final JPanel databaseFieldPanel = new JPanel();
   
   private void userLoginDialog() {

      // Login panel
      final JPanel authData = new JPanel();
      authData.setLayout(new GridLayout(0, 2));

      //Label and input fields for username and password
      authData.add(new JLabel("User name"));
      final JTextField usernameField = new JTextField(10);
      authData.add(usernameField);

      authData.add(new JLabel("Password"));
      final JPasswordField passwordField = new JPasswordField();
      authData.add(passwordField);
      
      // JOptionPane.showOptionDialog options
      String[] options = {"Login",// index 0
                        "Cancel"};// index 1

      if (0 == JOptionPane.showOptionDialog ( // 0 is "login" option index, see above
            null, authData, null, 0, 0, null, options, options[0])) {

         persistenceHandler.connect();
         if (persistenceHandler.isAuthorized(
               usernameField.getText(), passwordField.getPassword())) {
            setGuiConnectionState(true);
         } else {
            // Error message possibly be overwritten by subsequent disconnect() operation
            final String currentErrMsg = persistenceHandler.getErrorMessage();
            persistenceHandler.disconnect();
            JOptionPane.showMessageDialog(null, currentErrMsg);
         }
      }
   }

   private void setGuiConnectionState(final boolean state) {
      if (state) {
         toggleConnectButton.setText("Disconnect");
      } else {
         toggleConnectButton.setText("Connect");
      }
      for (final Component c: databaseFieldPanel.getComponents()){
         c.setEnabled(state);
      }
   }

   public static void main(String[] args) throws SQLException {
      InsertPerson app = new InsertPerson();
      app.setVisible(true);
   }
   
   public InsertPerson (){
      super ("Add a person's data");
      
      setSize(500, 500);
      
      addWindowListener(new WindowAdapter() {
         // In case a user closes our Jframe window while still being connected
         // we have to close the database connection.
         @Override
         public void windowClosing(WindowEvent e) {
            super.windowClosing(e);
            if (persistenceHandler.isConnected() && !persistenceHandler.disconnect()) {
               System.exit(1);
            } else {
               System.exit(0);
            }
         }
      });
      Box top = Box.createHorizontalBox();
      add(top, BorderLayout.NORTH);
      top.add(toggleConnectButton);
      
      toggleConnectButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed(ActionEvent e) {
            if (persistenceHandler.isConnected()) {
               if (persistenceHandler.disconnect()){
                  setGuiConnectionState(false);
               } else {
                  JOptionPane.showMessageDialog(null, persistenceHandler.getErrorMessage());
               }
            } else {
               userLoginDialog();
            }
         }
      });
            
      databaseFieldPanel.setLayout(new GridLayout(0, 3)); //Third column for validation label
      add(databaseFieldPanel);

      for (UserInputUnit unit: userInputUnits) {
         databaseFieldPanel.add(unit.label);
         databaseFieldPanel.add(unit.verifier.field);
         databaseFieldPanel.add(unit.verifier.validationLabel);
      }
      
      insertButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed(ActionEvent e) {
            if (inputValuesAllValid()) {
               if (persistenceHandler.add(
                     name.getText(),
                     email.getText(),
                     login.getText(),
                     passwordField.getPassword())) {
                  clearMask();
                  JOptionPane.showMessageDialog(null, "Succesfully inserted dataset");
               } else {
                  JOptionPane.showMessageDialog(null, persistenceHandler.getErrorMessage());
               }
            } else {
               JOptionPane.showMessageDialog(null, "Invalid input value(s)");
            }
         }
      });
      databaseFieldPanel.add(Box.createGlue());
      databaseFieldPanel.add(insertButton);   
      setGuiConnectionState(false);
      pack();
   }
   private void clearMask() {
      for (UserInputUnit unit: userInputUnits) {
         unit.verifier.field.setText("");
         unit.verifier.clear();
      }
   }
   private boolean inputValuesAllValid() {
      for (UserInputUnit unit: userInputUnits) {
         if (!unit.verifier.verify(unit.verifier.field)){
            return false;
         }
      }
      return true;
   }   
}
